
; External functions
EXTERN kSysExit

; External C functions
EXTERN fgets, fopen, fclose, feof, printf
EXTERN strchr, strlen, strtok, strcmp, atoi, strcpy
EXTERN inet_addr

; External data
EXTERN strFileNotFound, strTerminating, fOpenRead, fileConfig


; Exported symbols
GLOBAL loadConfig:FUNCTION
GLOBAL cfgBindPort:DATA, cfgBindIp:DATA, cfgDocumentRoot:DATA, cfgRoot:DATA


SECTION .data
cfgBindIp:     dd 127 << 0 | 0 << 8 | 0 << 16 | 1 << 24
cfgBindPort:   dd 808
cfgRoot:       db '/srv/www/local/'
    times 120  db 0

SECTION .data

var:
.fHandle    dd 0
.num        db 'num = %d', 10, 0
.pair       db '|%s = %s|', 10, 0
.hint       db ' hint = "%s"', 10, 0
.hintd      db ' hint = "%d"', 10, 0
.delimiter  db '= ', 0
.key        dd 0
.value      dd 0
.kPort      db 'port', 0
.kIp        db 'ip', 0
.kRoot      db 'root', 0



SECTION .bss 
buffLen:    equ 1024
buff:       times buffLen db 



SECTION .code 

; procedure load and parse configuration data
loadConfig:
    push dword fOpenRead    ; mode - read
    push dword fileConfig   ; config location
    CALL fopen              ; do fopen()
    add esp, 2 * 4          ; correct stack
    mov [var.fHandle], eax  ; var.fHandle = eax
    cmp eax, 0              ; eax ?
    jne .readFile           ; eax != 0 - that's ok!
    
.cantOpen:
    ; Output error message
    push dword fileConfig     
    push dword strFileNotFound 
    CALL printf             
    add esp, 2 * 4
    push dword strTerminating
    CALL printf
    add esp, 4
    CALL kSysExit       ; quit...
    jmp .endProc
    
.readFile:
.checkEof:  ; Check for end of file
    push dword [var.fHandle]  ; push file handle
    CALL feof           ; feof()
    add esp, 4          ; correct stack head
    test eax, eax       ; eax?
    jnz .closeFile      ; EOF?!
    
.getLine:   ; get 1 line from file
    push dword [var.fHandle]  ; push handle again
    push dword buffLen    ; buffer length
    push dword buff       ; buffer pointer
    CALL fgets          ; call fgets()
    add esp, 4 * 3      ; corrent stack
    test eax, eax       ; check for error
    jz .readFile        ; eax = 0 means error

.correctString:   ; Correct end of string - where '#' located
    push dword '#'      ; symbol
    push dword buff ; config string
    CALL strchr         ; call strchr()
    add esp, 4 * 2      ; stack correct
    cmp eax, 0          ; eax == 0 ?
    je .checkLength     ; so check length next
    
    mov [eax], byte 0  ; set end of string

.checkLength:  ; check for zero length
    push dword buff ; string from file
    CALL strlen         ; strlen()
    add esp, 4          ; esp+=4..
    test eax, eax       ; what about eax?
    jz .readFile        ; It's empty string!?

.removeChr10:    ; remove end-of string symbol
    push dword 10       ; symbol
    push dword buff ; config string
    CALL strchr         ; call strchr()
    add esp, 4 * 2      ; stack correct
    test eax, eax
    jz .isKeyValuePair
    
    mov [eax], byte 0

    
.isKeyValuePair:  ; Search for '=', which split name = value" pair
    push dword '='      ; symbol
    push dword buff ; config string
    CALL strchr         ; call strchr()
    add esp, 4 * 2      ; stack correct
    test eax, eax       ; eax?!
    jz .readFile        ; if eax == 0, read next line

        
.split: ; Splits pair 
    push dword var.delimiter  ; push addr to delimiter '='
    push dword buff       ; string from config file
    CALL strtok               ; strtok()
    add esp, 4 * 2            ; esp += 8
    mov [var.key], eax        ; key string

    
    push dword var.delimiter; string
    push dword 0            ; push NULL to retrive next token
    CALL strtok             ; strtok(NULL, delimiter)
    add esp, 4 * 2          ; esp += 8
    mov [var.value], eax    ; value string
    
.checkForPort: ; is key == port?
    push dword [var.key]    ; string 1
    push dword var.kPort    ; string 2
    CALL strcmp             ; do compare
    add esp, 4 * 2          ; fix stack
    test eax, eax           ; what about eax?
    jne .checkForIp         ; not equals? next check
    
    push dword [var.value]  ; addr of string with number
    CALL atoi               ; do calculate integer value
    add esp, 4              ; correct stack

    mov [cfgBindPort], ax  ; save value

    jmp .checkEnd           ; we are done


.checkForIp:   ; is key == ip?
    push dword [var.key]    ; string 1
    push dword var.kIp      ; string 2
    CALL strcmp             ; do strcmp()
    add esp, 4 * 2          ; correct stack
    test eax, eax           ; what about eax?
    jne .checkForRoot       ; not equals? next check

    push dword [var.value]  ; ip addr
    CALL inet_addr          ; do transform
    add esp, 4              ; correct stack

    mov [cfgBindIp], eax    ; save value

    
.checkForRoot:   ; is key == root?
    push dword [var.key]    ; string 1
    push dword var.kRoot    ; string 2
    CALL strcmp             ; do strcmp()
    add esp, 4 * 2          ; correct stack
    test eax, eax           ; what about eax?
    jne .checkEnd           ; not equals? next check
    
    push dword [var.value]  ; copy from
    push dword cfgRoot      ; copy to
    CALL strcpy             ; copy config value item
    add esp, 4 * 2          ; correct stack


.checkEnd:        
    jmp .readFile
    
    
    push dword [var.value]
    push dword [var.key]
    push dword var.pair
    call printf
    add esp, 4 * 3

    jmp .readFile       ; read next line
    
    
.closeFile:    
    push dword [var.fHandle]  ; file handle to close
    CALL fclose         ; make fclose()
    add esp, 4          ; stack..
    
.endProc:
    ret 
