EXTERN cfgBindIp, cfgBindPort, maxConnections, headerServer
EXTERN htons, shutdown, fprintf, printf, malloc, free, realloc, strlen
EXTERN kSocketCall, kClose, processRequest
GLOBAL sendServerHeader:FUNCTION

; Constants used for arguments(from kernel source)
%define AF_INET 2
%define IPPROTO_TCP 6
%define SOCK_STREAM 1
%define PF_INET 2
%define MSG_PEEK 2
%define MSG_WAITALL 100h
%define SOL_SOCKET      1
%define SO_REUSEADDR    2

; Function numbers
%define SYS_SOCKET 1
%define SYS_BIND   2
%define SYS_LISTEN 4
%define SYS_ACCEPT 5
%define SYS_SEND   9
%define SYS_RECV   10
%define SYS_SETSOCKOPT  14




; Step increase memory for receive buffer
%define REALLOC_STEP 64

SECTION .data

connect            db    "We have a connection!", 10, 0               ; First string...
hello            db    "Hello... =]", 10, 0                           ; Second string...
goodbye            db    "Goodbye... =[", 0                           ; Third string...
socksbroke        db    "ERROR: socket() failed!", 10, 0           ; Fourth string...
bindsbroke        db    "ERROR: bind() failed!", 10, 0               ; Fifth string...
listensbroke    db    "ERROR: listen() failed!", 10, 0           ; Sixth string...
acceptsbroke    db    "ERROR: accept() failed!", 10, 0           ; Seventh string...
retval          db     "value buffer: %d, recv: %d", 10, 0
retstr          db      "recived: %s", 10 , 0
html            db  "HTTP 1.1", 10, 0, '$'
html_len        equ $ - html
one             dd  1


sa: ; sockaddr_in structure
.sin_family dw    0
.sin_port   dw    0
.sin_addr   dd    0

sock_args: ; socket() function argument
    dd PF_INET, SOCK_STREAM, IPPROTO_TCP


bind_args: ; bind() function arguments
    .fd         dd 0    ; socket handle
    .sockaddr   dd sa   ; pointer to sockaddr structure
    .socklen_t  dd 16   ; socklen_t addrlen

listen_args: ; listen() function arguments
    .sock       dd 0              ; sock handle
    .backlog    dd maxConnections ; max connections
    
accept_args: ; accept() arguments   
    .sockfd     dd 0       ; socket handle
    .addr       dd 0       ; struct sockaddr *addr
    .addrlen    dd 0       ; socklen_t *addrlen
    
sockopts_args: ; setsockopt() and getsockopt() args
    .sockfd     dd 0            ; socket handle
    .level      dd SOL_SOCKET   ; manipulation level
    .optname    dd SO_REUSEADDR ; option in selected level
    .optval     dd one          ; option value
    .optlen     dd $ - one      ; size of option value
    
socketfd: dd 0  ; socket handle
    
section .code                      ; Section Declaration
GLOBAL sock

sock:
    
;    mov ebp, esp
    
;int socket(int domain, int type, int protocol);
.doSocket:
    mov ebx, SYS_SOCKET                     ; socket() = int call 1
    mov ecx, dword sock_args       ; arguments to socket()
    CALL kSocketCall    
    mov [socketfd], eax            ; save socket handle
                               ;
    cmp eax, -1
    je sockerr

;int setsockopt(int s, int level, int optname, 
;       const void *optval, socklen_t optlen);
.setSockOpt:
    ; ignoring port in state TIME_WAIT
    mov ebx, SYS_SETSOCKOPT
    mov eax, dword [socketfd]
    mov [sockopts_args.sockfd], eax
    mov ecx, dword sockopts_args
    CALL kSocketCall
    
    cmp eax, -1
    je sockerr

;int bind(int socket, const struct sockaddr *address,
;       socklen_t address_len);
.doBind:
    ; fillup sockaddr
    mov [sa.sin_family], word AF_INET ; address family
    mov al, byte [cfgBindPort+1]      ; reverse bytes hi=>lo
    mov ah, byte [cfgBindPort]        ; reverse bytes lo=>hi
    mov [sa.sin_port], ax             ; set port
    mov eax, dword [cfgBindIp]        ; get ip from config
    mov [sa.sin_addr], eax            ; set ip to structure

    ; form arguments
    mov eax, dword [socketfd]   ; copy socket
    mov [bind_args.fd], eax     ; handle to struct 
    
    mov ebx, SYS_BIND   ; require bind()
    mov ecx, bind_args  ; bind arguments
    CALL kSocketCall
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Do a little error-echecking here...
    cmp eax, -1
    je binderr

; int listen(int socket, int backlog);
.doListen:
    mov eax, [socketfd]         ; copy socket handle
    mov [listen_args.sock], eax ; to structure listen_args
    mov ebx, SYS_LISTEN         ; require listen()
    mov ecx, listen_args        ; arguments
    CALL kSocketCall
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Do a little error-echecking here...
    cmp eax, -1
    je listenerr


;accept(fd, NULL, 0);
.doAccept:  
    mov eax, [socketfd]           ; copy socket handle
    mov [accept_args.sockfd], eax ; to structure

    mov ebx, SYS_ACCEPT    ; we need accept() function
    mov ecx, accept_args   ; ptr to arguments
    CALL kSocketCall

; Do a little error-echecking here...
    cmp eax, -1
    je accepterr
    
.processConnection:  ; here we have a connected socket peer in eax
    push eax
    CALL connectionHandler
    
    jmp .doAccept
                               ;
;exit_s(0)
exit_s:
 ;   mov esp, ebp
    ret 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; These are just to print an error message to stdout should
; one of our socketcall()'s fail.
sockerr:
    mov eax, 4
    mov ebx, 1
    mov ecx, socksbroke
    mov edx, 24
    int 0x80
    jmp exit_s

binderr:
    mov eax, 4
    mov ebx, 1
    mov ecx, bindsbroke
    mov edx, 22
    int 0x80
    jmp exit_s

listenerr:
    mov eax, 4
    mov ebx, 1
    mov ecx, listensbroke
    mov edx, 24
    int 0x80
    jmp exit_s

accepterr:
    mov eax, 4
    mov ebx, 1
    mov ecx, acceptsbroke
    mov edx, 24
    int 0x80
    jmp exit_s
    

; connection handler function
; get 1 arg - socket handle
connectionHandler:
    push ebp
    mov ebp, esp    ; preserve esp
    %define sock dword [ebp+8]  ; connected socket
    jmp .initVars

.vars: ; declare variables (allocate memory for each)
    ; begin recv_args structure (argument for recv function)
    %define recv_args 8 ; main offset from esp
    %define recv_args_sock   esp+recv_args
    %define recv_args_buffer esp+4+recv_args
    %define recv_args_length esp+8+recv_args
    %define recv_args_flags  esp+12+recv_args
    sub esp, 16 ; allocate memory for recv_args
    %define send_args 16
    %define send_args_sock 16

.initVars: 
    mov eax, sock   
    mov [recv_args_sock], eax
    mov [recv_args_flags], dword MSG_PEEK
    mov [recv_args_length], dword REALLOC_STEP

    
    
.code:

.alloc:
    push dword [recv_args_length]; count of bytes we need
    CALL malloc                 ; allocate memory from heap
    add esp, 4                  ; correct stack
    mov [recv_args_buffer], eax ; save pointer to buffer

        
.recvLength:   
    mov ebx, SYS_RECV   ; we need recv()
    mov ecx, esp
    add ecx, recv_args  ; arguments
    CALL kSocketCall    
   
    cmp eax, dword -11        ; try again code?
    je .recvLength            ; recieve again

    mov edx, eax
    mov ebx, dword [recv_args_length]
    push edx
    push ebx
    push retval
    call printf
    add esp, 8
    pop eax

      
    cmp eax, [recv_args_length]
    jb .recv

.realoc:
    add [recv_args_length], dword REALLOC_STEP
    mov eax, dword [recv_args_buffer]
    push dword [recv_args_length]
    push eax
    CALL realloc
    add esp, 2 * 4
    mov [recv_args_buffer], eax
           
    jmp .recvLength
    
.recv:

    mov ebx, SYS_RECV   ; we need recv()
    mov [recv_args_flags], dword 0 ; without flags!
    mov ecx, esp                ; point ecx ..
    add ecx, recv_args          ; .. to arguments
    CALL kSocketCall            ; do recv()
    mov [recv_args_length], eax ; store received message length
    
    cmp eax, 0         ; is there any error?
    jbe .closeSocket   ; then cancel request




         
    push dword [recv_args_buffer]
    push dword sock
    CALL processRequest         ; let's process request!
    
.closeSocket:    
    push sock
    call kClose

.freeMem:
    mov eax, dword [recv_args_buffer]
    push eax
    CALL free
    add esp, 8
    mov [recv_args_length], dword 0   ; mark block as free

.return:
    cmp [recv_args_length], dword 0   ; we used malloc?
    jne .freeMem    ; yeah, so we need free memory

    mov esp, ebp    ; restore stack poiter
    pop ebp
    ret 4
   

;write(1, "We have a connection!", 22);
    mov eax, 4                   ; write() syscall
    mov ebx, 1                   ; stdout
    mov ecx, connect           ; our string
    mov edx, 22               ; 22 characters in length
    int 0x80                   ; Call the kernel
                               ;
;write(fd, "Hello... =]", 12);
    mov eax, 4                   ; write() syscall
    mov ebx, edi               ; sockfd
    mov ecx, hello               ; our string to send
    mov edx, 12               ; 12 characters in length
    int 0x80                   ; Call the kernel
    ;push dword hello
    ;push dword edi
    ;call fprintf
    ;add esp, 8
                               ;
;write(fd, "Goodbye... =[", 14);
    mov eax, 4                   ; write() syscall
    mov ebx, edi               ; sockfd
    mov ecx, goodbye           ; our string to send
    mov edx, 14               ; 14 charactes in length
    int 0x80                   ; Call the kernel
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Now we clean up (Close the open file descriptor/s etc.) and exit_s
    push dword edi
    Call shutdown
    pop eax

;close(fd)
    push edi
    CALL kClose
    
    mov eax, 0
    ret 




; send to socket 'Server' header
; first argument in stack - sock handle
sendServerHeader:
    push ebp
    mov ebp, esp
    
    push dword headerServer    ; string to send
    call strlen                 ; get string length
    add esp, 4                  ; correct stack
        
    push dword 0                ; flags
    push eax                    ; length
    push dword headerServer     ; header string
    push dword [ebp+8]          ; socket handle
    mov ecx, esp                ; ptr to struct in stack
    mov ebx, 9                  ; send() function number
    call kSocketCall            ; call it
    add esp, 4 * 4              ; correct stack space

    mov esp, ebp    ; restore base stack pointer
    pop ebp         
    
    ret 4
